with Ada.Text_IO;

with Names;
with ModeMgr;
with UnitMgr;
with HW_Simulation;


package body AocsMgr is

   use Names;

   Units : Units_T renames UnitMgr.Units;


   procedure Sensor_Data_Processing (UnId : Unit_ID_T)
   is
      Ret_Value_from_Call : Boolean;
   begin
-- AocsMgr.Main execution statement    4
-- AocsMgr.Main execution statement   27
      Ret_Value_from_Call := HW_Simulation.Get_Sensor_Data (
         UnId, Units(UnId).CuBrId);
-- AocsMgr.Main execution statement    7
-- AocsMgr.Main execution statement   30
      if not Ret_Value_from_Call then
-- AocsMgr.Main execution statement    8
-- AocsMgr.Main execution statement   31
         Units(UnId).Br(Units(UnId).CuBrId).Error := Invalid_Data;
      end if;
   end Sensor_Data_Processing;

   procedure Actuator_Commanding (UnId : Unit_ID_T)
   is
      Ret_Value_from_Call : Boolean;
   begin
-- AocsMgr.Main execution statement   19
      Ret_Value_from_Call := HW_Simulation.Receive_Command (
         UnId, Units(UnId).CuBrId);
-- AocsMgr.Main execution statement   22
      if not Ret_Value_from_Call then
-- AocsMgr.Main execution statement   23
         Units(UnId).Br(Units(UnId).CuBrId).Error := Commanding_Failure;
      end if;
   end Actuator_Commanding;


   procedure Execute_Currently_Selected_Controller is
   begin

-- AocsMgr.Main execution statement   10
      if Control_Law.Current /= None then

-- AocsMgr.Main execution statement   11
         if Control_Law.Status = Preparing
            and Control_Law.Timer > 10
         then

-- AocsMgr.Main execution statement   12
            Control_Law.Status := Ready;

         end if;

-- AocsMgr.Main execution statement   13
         Control_Law.Timer := Control_Law.Timer + 1;

-- AocsMgr.Main execution statement   14
         if Control_Law.Current /= Virtual
            and HW_Simulation.Spurious_Error_Injection
         then
            Ada.Text_IO.Put_Line ("AOCS ERR INJECTED c="
               & Control_Law.Current'Img);

-- AocsMgr.Main execution statement   15
            Control_Law.Error := Attitude_Error;

         end if;

      end if;

   end Execute_Currently_Selected_Controller;


   procedure PLI_Management is
      Ret_Value_from_Call : Boolean;
   begin
      Ada.Text_IO.Put_Line ("PLI MAN br="
         & Units (PLI).CuBrId'Img
         & " t=" & Control_Law.PLI_Timer'Img
         & " r=" & PLI_Is_Ready'Img);

-- AocsMgr.Main execution statement   26
      Sensor_Data_Processing (PLI);

-- AocsMgr.Main execution statement   32
      Control_Law.PLI_Timer := Control_Law.PLI_Timer + 1;

      -- ERROR INJECTION CODE:
      --
-- AocsMgr.Main execution statement   33
      if Control_Law.PLI_Timer > 200 then
-- AocsMgr.Main execution statement   34
         Ret_Value_from_Call := PLI_Is_Ready;
-- AocsMgr.Main execution statement   42
         if Ret_Value_from_Call then
-- AocsMgr.Main execution statement   43
            Ret_Value_from_Call := HW_Simulation.Spurious_LOA_Error (
               PLI, Units (PLI).CuBrId);
-- AocsMgr.Main execution statement   46
            if Ret_Value_from_Call then
                Ada.Text_IO.Put_Line ("PLI ERR INJECTED br="
                       & Units (PLI).CuBrId'Img);
-- AocsMgr.Main execution statement   47
                Units(PLI).Br(Units(PLI).CuBrId).Error := Loss_Of_Accuracy;
            end if;
         end if;
      end if;

   end PLI_Management;


   procedure Main is
   begin

-- AocsMgr.Main execution statement    1
      for UnId in Unit_ID_T loop

-- AocsMgr.Main execution statement    2
         if Unit_Type (UnId) = Sensor
            and Units(UnId).Br(Units(UnId).CuBrId).Status = Locked
         then

-- AocsMgr.Main execution statement    3
            Sensor_Data_Processing (UnId);

         end if;

-- AocsMgr.Main execution statement 1001
      end loop;

-- AocsMgr.Main execution statement    9
      Execute_Currently_Selected_Controller;
      -- Performs the computation of the attitude control algorithms

-- AocsMgr.Main execution statement   16
      for UnId in Unit_ID_T loop

-- AocsMgr.Main execution statement   17
         if Unit_Type (UnId) = Actuator
            and Units(UnId).Br(Units(UnId).CuBrId).Status = Locked
         then

-- AocsMgr.Main execution statement   18
            Actuator_Commanding (UnId);

         end if;

-- AocsMgr.Main execution statement 1016
      end loop;

-- AocsMgr.Main execution statement   24
      if Units(PLI).Br(Units(PLI).CuBrId).Status = Locked
         and Control_Law.PLI_Enabled
      then

-- AocsMgr.Main execution statement   25
         PLI_Management;

      end if;

   end Main;


   procedure Stop_PLI_Management is
   begin

-- ModeMgr.Main execution statement   48
-- ModeMgr.Main execution statement   87
-- ModeMgr.Main execution statement  232
-- ModeMgr.Main execution statement  330
-- ModeMgr.Main execution statement  424
      Control_Law.PLI_Enabled := False;

   end Stop_PLI_Management;


   procedure Start_PLI_Management is
   begin

-- ModeMgr.Main execution statement  516
      Control_Law.PLI_Enabled := True;

-- ModeMgr.Main execution statement  517
      Control_Law.PLI_Timer := 0;

   end Start_PLI_Management;


   function PLI_Is_Ready return Boolean
   is
      Ret_Value : Boolean;
      Ret_Value_from_Call : Boolean;
   begin

-- AocsMgr.Main execution statement   35
-- ModeMgr.Main execution statement   28
      Ret_Value := (
         Units(PLI).Br(Units(PLI).CuBrId).Error = None
         and Units(PLI).Br(Units(PLI).CuBrId).Status = Locked
         and (Units(PLI).Br(Units(PLI).CuBrId).State = PLI_Standby
              or Units(PLI).Br(Units(PLI).CuBrId).State = PLI_Science)
         and Control_Law.PLI_Enabled);
-- AocsMgr.Main execution statement   36
-- ModeMgr.Main execution statement   29
      if Ret_Value then
-- AocsMgr.Main execution statement   37
-- ModeMgr.Main execution statement   30
         Ret_Value_from_Call := Fine_Pointing_Reached;
-- AocsMgr.Main execution statement   40
-- ModeMgr.Main execution statement   33
         Ret_Value := (Ret_Value_from_Call
            and Control_Law.PLI_Timer > 100 );
      end if;
-- AocsMgr.Main execution statement   41
-- ModeMgr.Main execution statement   34
      return Ret_Value;

   end PLI_Is_Ready;


   procedure Select_Controller (Algo : Controller_T) is
   begin

-- ModeMgr.Main execution statement   43
-- ModeMgr.Main execution statement   78
-- ModeMgr.Main execution statement   82
-- ModeMgr.Main execution statement  194
-- ModeMgr.Main execution statement  227
-- ModeMgr.Main execution statement  297
      Control_Law.Error := None;

-- ModeMgr.Main execution statement   44
-- ModeMgr.Main execution statement   83
-- ModeMgr.Main execution statement  195
-- ModeMgr.Main execution statement  228
-- ModeMgr.Main execution statement  298
-- ModeMgr.Main execution statement  574
      Control_Law.Current := Algo;

-- ModeMgr.Main execution statement   45
-- ModeMgr.Main execution statement   84
-- ModeMgr.Main execution statement  196
-- ModeMgr.Main execution statement  229
-- ModeMgr.Main execution statement  299
-- ModeMgr.Main execution statement  575
      Control_Law.Status := Idle;

-- ModeMgr.Main execution statement   46
-- ModeMgr.Main execution statement   85
-- ModeMgr.Main execution statement  197
-- ModeMgr.Main execution statement  230
-- ModeMgr.Main execution statement  300
-- ModeMgr.Main execution statement  576
      Control_Law.Timer := 0;

   end Select_Controller;


   procedure Prepare_Controller is
   begin

-- ModeMgr.Main execution statement  199
-- ModeMgr.Main execution statement  302
      Control_Law.Status := Preparing;

-- ModeMgr.Main execution statement  200
-- ModeMgr.Main execution statement  303
      Control_Law.Timer := 0;

   end Prepare_Controller;


   function Controller_Is_Ready return Boolean
   is
      Ret_Value : Boolean;
   begin

-- ModeMgr.Main execution statement  203
-- ModeMgr.Main execution statement  306
      Ret_Value := (Control_Law.Status = Ready);
-- ModeMgr.Main execution statement  204
-- ModeMgr.Main execution statement  307
      return Ret_Value;

   end Controller_Is_Ready;


   procedure Start_Controller is
   begin

-- ModeMgr.Main execution statement  215
-- ModeMgr.Main execution statement  318
      Control_Law.Status := Running;

-- ModeMgr.Main execution statement  216
-- ModeMgr.Main execution statement  319
      Control_Law.Timer := 0;

   end Start_Controller;


   function Coarse_Pointing_Reached return Boolean is
      Ret_Value : Boolean;
   begin

-- ModeMgr.Main execution statement   18
      Ret_Value := (
         Control_Law.Current = Coarse_Pointing_Control
         and Control_Law.Status = Running
         and Control_Law.Timer > 2000 );
-- ModeMgr.Main execution statement   19
      return Ret_Value;

   end Coarse_Pointing_Reached;


   function Fine_Pointing_Reached return Boolean is
      Ret_Value : Boolean;
   begin

-- AocsMgr.Main execution statement   38
-- ModeMgr.Main execution statement   23
-- ModeMgr.Main execution statement   31
      Ret_Value := (
         Control_Law.Current = Fine_Pointing_Control
         and Control_Law.Status = Running
         and Control_Law.Timer > 4000 );
-- AocsMgr.Main execution statement   39
-- ModeMgr.Main execution statement   24
-- ModeMgr.Main execution statement   32
      return Ret_Value;

   end Fine_Pointing_Reached;


end AocsMgr;
