with Ada.Text_IO;

with Names;
with AocsMgr;
with FdirMgr;
with UnitMgr;
with ModeMgr.Transitions;


package body ModeMgr is

   use Names;


   procedure Init_Transition is
   begin

-- FdirMgr.Main execution statement   51
-- FdirMgr.Main execution statement   67
-- ModeMgr.Main execution statement  559
-- ModeMgr.Main execution statement  569
      Trans.Target := None;
-- FdirMgr.Main execution statement   52
-- FdirMgr.Main execution statement   68
-- ModeMgr.Main execution statement  560
-- ModeMgr.Main execution statement  570
      Trans.Done   := False;
-- FdirMgr.Main execution statement   53
-- FdirMgr.Main execution statement   69
-- ModeMgr.Main execution statement  561
-- ModeMgr.Main execution statement  571
      Trans.Step   := 0;
-- FdirMgr.Main execution statement   54
-- FdirMgr.Main execution statement   70
-- ModeMgr.Main execution statement  562
-- ModeMgr.Main execution statement  572
      Trans.Timer  := 0;

   end Init_Transition;


   Separation_Counter : Natural := 0;

   procedure Separation_StartCheck is
   begin

-- ModeMgr.Main execution statement  620
      if Separation_Counter = 0 then

-- ModeMgr.Main execution statement  621
         Separation_Counter := 1;

      end if;

   end Separation_StartCheck;


   function Separation_Done return Boolean is
      Ret_Value : Boolean;
   begin

-- ModeMgr.Main execution statement    8
      if Separation_Counter > 0 then

-- ModeMgr.Main execution statement    9
         if Separation_Counter > 10 then

-- ModeMgr.Main execution statement   10
            Ret_Value := True;
-- ModeMgr.Main execution statement   11
            return Ret_Value;

         end if;

-- ModeMgr.Main execution statement   12
         Separation_Counter := Separation_Counter + 1;

      end if;

-- ModeMgr.Main execution statement   13
      Ret_Value := False;
-- ModeMgr.Main execution statement   14
      return Ret_Value;

   end Separation_Done;


   procedure Start_Possible_Autonomous_Transition is
      Ret_Value_from_Call : Boolean;
   begin

-- ModeMgr.Main execution statement    3
      case Mode is

         when Off =>

-- ModeMgr.Main execution statement    4
            if not Trans.Standby_Entered_Once then
               -- Off entered after CDMU boot,
               -- force autonomous start of transition to Standby

-- ModeMgr.Main execution statement    5
               Trans.Standby_Entered_Once := True;

-- ModeMgr.Main execution statement    6
               Trans.Target := Standby;

               -- else
               -- Off entered from higher mode means
               -- that CDMU reboot is already requested
               -- and attitude control is stopped.
            end if;


         when Standby =>

-- ModeMgr.Main execution statement    7
            Ret_Value_from_Call := Separation_Done;
-- ModeMgr.Main execution statement   15
            if Ret_Value_from_Call then

-- ModeMgr.Main execution statement   16
               Trans.Target := Safe;

            end if;


         when Safe =>

-- ModeMgr.Main execution statement   17
            Ret_Value_from_Call := AocsMgr.Coarse_Pointing_Reached;
-- ModeMgr.Main execution statement   20
            if Ret_Value_from_Call then

-- ModeMgr.Main execution statement   21
               Trans.Target := Nominal;

            end if;


         when Nominal =>

-- ModeMgr.Main execution statement   22
            Ret_Value_from_Call := AocsMgr.Fine_Pointing_Reached;
-- ModeMgr.Main execution statement   25
            if Ret_Value_from_Call then

-- ModeMgr.Main execution statement   26
               Trans.Target := Preparation;

            end if;


         when Preparation =>

-- ModeMgr.Main execution statement   27
            Ret_Value_from_Call := AocsMgr.PLI_Is_Ready;
-- ModeMgr.Main execution statement   35
            if Ret_Value_from_Call then

-- ModeMgr.Main execution statement   36
               Trans.Target := Science;

            end if;


         when Science =>

            null;
            -- Stay in Science mode until FDIR forces back to some
            -- earlier mode

      end case;

   end Start_Possible_Autonomous_Transition;


   procedure Start_FDIR_Transition (Target_Mode : Target_Mode_T) is
   begin

      -- Abort possible on-going transition:
-- FdirMgr.Main execution statement   50
-- FdirMgr.Main execution statement   66
      Init_Transition;

-- FdirMgr.Main execution statement   55
-- FdirMgr.Main execution statement   71
      Trans.FDIR_Request := Target_Mode;

-- FdirMgr.Main execution statement   56
-- FdirMgr.Main execution statement   72
      Trans.Target := Target_Mode;

   end Start_FDIR_Transition;


   procedure Set_FDIR_Request (Target_Mode : Target_Mode_T)
   is
      -- Invariant: Target_Mode /= Standby and Target_Mode /= None
      -- Invariant: Target_Mode < Mode
      --            or (Mode = Safe and Target_Mode = Safe)
      Chosen_Target_Mode : Target_Mode_T;
   begin

-- FdirMgr.Main execution statement   44
-- FdirMgr.Main execution statement   60
      if Trans.FDIR_Request = None then
         -- No on-going old FDIR request

-- FdirMgr.Main execution statement   45
-- FdirMgr.Main execution statement   61
         Chosen_Target_Mode := Target_Mode;

      -- Error while recovery is already running:
-- FdirMgr.Main execution statement   46
-- FdirMgr.Main execution statement   62
      elsif Target_Mode = Off
         or Trans.Target = Off
         or Trans.Target = Standby
         or Trans.Target = Safe
         or Trans.FDIR_Request = Off
         or Trans.FDIR_Request = Safe
      then

-- FdirMgr.Main execution statement   47
-- FdirMgr.Main execution statement   63
         Chosen_Target_Mode := Off;

      else

-- FdirMgr.Main execution statement   48
-- FdirMgr.Main execution statement   64
         Chosen_Target_Mode := Safe;

      end if;

-- FdirMgr.Main execution statement   49
-- FdirMgr.Main execution statement   65
      Start_FDIR_Transition (Chosen_Target_Mode);

   end Set_FDIR_Request;


   procedure Request_CDMU_Reboot is
   begin

-- ModeMgr.Main execution statement   42
      AocsMgr.Select_Controller (AocsMgr.None);

-- ModeMgr.Main execution statement   47
      AocsMgr.Stop_PLI_Management;

-- ModeMgr.Main execution statement   49
      UnitMgr.Release_All;

-- ModeMgr.Main execution statement   73
      Trans.Emergency_Reboot_Requested := True;

   end Request_CDMU_Reboot;


   procedure Execute_Step is
   begin

-- ModeMgr.Main execution statement   40
      case Trans.Target is

         when Off =>
            -- Emergency transition to Off

-- ModeMgr.Main execution statement   41
            Request_CDMU_Reboot;
-- ModeMgr.Main execution statement   74
            Trans.Done := True;


         when Standby =>

-- ModeMgr.Main execution statement   75
            Transitions.To_StandbyMode;

         when Safe =>
-- ModeMgr.Main execution statement   79
            Transitions.To_SafeMode;


         when Nominal =>

-- ModeMgr.Main execution statement  223
            if Mode = Safe then

-- ModeMgr.Main execution statement  224
               Transitions.To_NominalMode;

-- ModeMgr.Main execution statement  326
            elsif Mode /= Off
               and Mode /= Standby
            then

-- ModeMgr.Main execution statement  327
               Transitions.Back_To_Nominal;

            end if;


         when Preparation =>

-- ModeMgr.Main execution statement  421
            Transitions.To_PrepMode;


         when Science =>

-- ModeMgr.Main execution statement  519
            Transitions.To_ScienceMode;

         when None =>

            null;

      end case;

   end Execute_Step;


   procedure Main
   is
      Prev_Step : Natural;
   begin

-- ModeMgr.Main execution statement    1
      if Trans.Target = None then
         -- No old transition on-going, and no new requests
         --
         -- Evaluate whether an autonomous transition should be started

-- ModeMgr.Main execution statement    2
         Start_Possible_Autonomous_Transition;

      end if;

      Ada.Text_IO.Put_Line ("trtarg="&Trans.Target'Img);
-- ModeMgr.Main execution statement   37
      if Trans.Target /= None then

-- ModeMgr.Main execution statement   38
         Prev_Step := Trans.Step;

-- ModeMgr.Main execution statement   39
         Execute_Step;
         -- Selects the procedure corresponding Trans.Current.
         -- The transition step counter is separately maintained.
         -- Here we continue execution of a transition.
         -- The step condition is always evaluated, and if it is true,
         -- the step activities are performed and step counter updated.

-- ModeMgr.Main execution statement  556
         if Trans.Done then
            -- If the last step is reached, time to change the mode

-- ModeMgr.Main execution statement  557
            Mode := Trans.Target;
            -- The resulting mode can be deduced from the transition (name)

-- ModeMgr.Main execution statement  558
            Init_Transition;

-- ModeMgr.Main execution statement  563
            Trans.FDIR_Request := None;

-- ModeMgr.Main execution statement  564
         elsif Prev_Step < Trans.Step then
            -- We have progress

-- ModeMgr.Main execution statement  565
            Trans.Timer := 0;

         else

-- ModeMgr.Main execution statement  566
            Trans.Timer := Trans.Timer + 1;

-- ModeMgr.Main execution statement  567
            if Trans.Timer > Trans.Timeout_Limit then

-- ModeMgr.Main execution statement  568
               Init_Transition;

-- ModeMgr.Main execution statement  573
               Trans.Error := Timeout;

            end if;

         end if;

      end if;

   end Main;


end ModeMgr;
